@@ -28,6 +28,58 @@
  *                 Derek J Smithies (derek@indranet.co.nz)
  *
  * $Log: main.h,v $
+ * Revision 1.97  2003/05/14 02:49:52  dereksmithies
+ * Add videolose option, so X percentage of video packets are dropped when in --videotest mode
+ *
+ * Revision 1.96  2003/05/07 02:45:58  dereks
+ * Alter ohphone to use the PSDLVideoOutputDevice class, which is now part of pwlib.
+ *
+ * Revision 1.95  2002/12/16 09:27:25  robertj
+ * Added new video bit rate control, thanks Walter H. Whitlock
+ *
+ * Revision 1.94  2002/10/31 00:53:38  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.93  2002/10/15 10:46:43  rogerh
+ * Add tweak by Sahai to support operation behind a NAT that forwards ports.
+ * This uses TranslateTCPAddress to change the address reported to connections
+ * that occur from outside the 192.x.x.x subnet. Copied from OpenMCU.
+ *
+ * Revision 1.92  2002/08/07 00:46:42  dereks
+ * Report statistics for video RTP packets. (No output if no video session)
+ *
+ * Revision 1.91  2002/08/05 01:33:34  robertj
+ * Fixed incorrect override of the SetupTransfer() function that stopped the
+ *   ability to do H.250 call transfer, thanks Vladimir Toncar
+ *
+ * Revision 1.90  2002/05/06 02:55:47  dereks
+ * Fix operation of --autodisconnect n , so works when video enabled.
+ *
+ * Revision 1.89  2002/04/26 03:33:32  dereks
+ * Major upgrade. All calls to SDL library are now done by one thread.
+ *
+ * Revision 1.88  2002/04/18 07:46:28  craigs
+ * Added new functions to remove H245 in SETUP, and to allow removal of
+ * user input indication capabilities
+ *
+ * Revision 1.87  2002/01/28 10:43:57  rogerh
+ * Wrap some code with HAS_LIDDEVICE so it compiles on systems without
+ * IxJ and VPBlaster
+ *
+ * Revision 1.86  2002/01/24 07:42:37  robertj
+ * Added ability to turn off H.245 negotiations in SETUP option
+ *
+ * Revision 1.85  2002/01/15 05:50:07  craigs
+ * Added support for VoIPBlaster
+ *
+ * Revision 1.84  2002/01/14 03:21:12  robertj
+ * Added ability to set capture caolour format, thanks Walter Whitlock
+ *
+ * Revision 1.83  2002/01/04 04:17:52  dereks
+ * Add code from Walter Whitlock to flip video, and improve the
+ * --videotest mode, which displays local video (without a call).  Many thanks.
+ *
  * Revision 1.82  2001/10/23 02:21:39  dereks
  * Initial release CU30 video codec.
  * Add --videotest option, to display raw video, but not invoke a call.
@@ -298,11 +350,21 @@
 
 #include <h323.h>
 
+#ifdef _WIN32
+#include <vblasterlid.h>
+#endif
+
 #ifdef HAS_IXJ
 #include <ixjlid.h>
 #endif
 
+#if defined(HAS_IXJ) || defined(HAS_VBLASTER)
+#define HAS_LIDDEVICE
+#endif
 
+#if P_SDL
+class PSDLDisplayThread;
+#endif
 
 #ifdef __BEOS__
 
@@ -310,6 +372,7 @@
 #include <Application.h>
 #endif
 
+
 class OhPhoneApplication : public BApplication 
 {
   public:
@@ -340,12 +403,14 @@ class CallOptions : public PObject
     BOOL Initialise(PArgList & args);
     void PrintOn(ostream & str) const;
 
-    BOOL   noFastStart;
-    BOOL   noH245Tunnelling;
-    BOOL   noSilenceSuppression;
-    int    jitter;
-    PINDEX connectRing;
-    WORD   connectPort;
+    BOOL     noFastStart;
+    BOOL     noH245Tunnelling;
+    BOOL     noH245InSetup;
+    BOOL     noSilenceSuppression;
+    unsigned minJitter;
+    unsigned maxJitter;
+    PINDEX   connectRing;
+    WORD     connectPort;
 };
 
 
@@ -361,7 +426,9 @@ class MyH323EndPoint : public H323EndPoint
     virtual void OnConnectionCleared(H323Connection &, const PString &);
     virtual BOOL OpenAudioChannel(H323Connection &, BOOL, unsigned, H323AudioCodec &);
     virtual BOOL OpenVideoChannel(H323Connection &, BOOL, H323VideoCodec &);
-    virtual H323Connection * SetupTransfer(const PString &, const PString &, const PString &, PString &);
+    virtual H323Connection * SetupTransfer(const PString &, const PString &, const PString &, PString &, void * userData = NULL);
+    virtual void TranslateTCPAddress(PIPSocket::Address &localAddr, const PIPSocket::Address &remoteAddr);
+
 
     // new functions
     BOOL Initialise(PConfigArgs & args, int verbose, BOOL hasMenu);
@@ -375,9 +442,11 @@ class MyH323EndPoint : public H323EndPoint
     void StopRinging();
     void HandleRinging();
 
-    void TestVideoGrabber();
+    void TestVideoGrabber(PConfigArgs &args);
+    void TestHandleUserInterface();
 
-#ifdef HAS_IXJ
+
+#ifdef HAS_LIDDEVICE
     void HandleStateChange(BOOL onHook);
     void HandleHandsetOffHook();
     void HandleHandsetOnHook();
@@ -398,14 +467,23 @@ class MyH323EndPoint : public H323EndPoint
 
     BOOL       terminateOnHangup;
 
+    void WaitForSdlTermination();
+    BOOL InitialiseSdl(PConfigArgs & args);
+
+    int GetChannelsOpenLimit() const
+      { return channelsOpenLimit; }
+
   protected:
     // only used internally
     BOOL SetSoundDevice(PConfigArgs &, const char *, PSoundChannel::Directions);
     void AnswerCall(H323Connection::AnswerCallResponse response);
 
+    void ReportSessionStatistics(H323Connection *connection, unsigned sessionID);
+
     CallOptions defaultCallOptions;
     CallOptions currentCallOptions;
 
+
     int  verbose;
     BOOL autoAnswer;
 
@@ -419,6 +497,7 @@ class MyH323EndPoint : public H323EndPoint
     PString videoReceiveDevice;
     int  videoQuality;
     int  videoTxQuality;
+    int  videoTxMinQuality;
 
     int  videoSize;       //0=small, 1=large.
     int  videoInput;
@@ -426,15 +505,19 @@ class MyH323EndPoint : public H323EndPoint
     int  videoFill;
     int  videoFramesPS;
     int  videoBitRate;
+    int  frameTimeMs;
 
     BOOL    videoIsPal;
     BOOL    videoFake;    
     BOOL    videoLocal;
     BOOL    videoPIP;
     PString videoDevice;
+    PString pfdColourFormat;
     int     videoCu30Stats;
     PString configDirectory;
 
+// used for testing the video (no connection made).
+    PVideoChannel *localVideoChannel; 
 
     PString autoDial;
     BOOL    dialAfterHangup;
@@ -480,12 +563,22 @@ class MyH323EndPoint : public H323EndPoint
     PSyncPoint exitFlag;
     BOOL       speakerphoneSwitch;
     BOOL       hasMenu;
-
-#ifdef HAS_IXJ
-    OpalIxJDevice xJack;
+#ifdef HAS_LIDDEVICE
     BOOL autoHook;
+
+    OpalLineInterfaceDevice * lidDevice;
+    BOOL isXJack;
 #endif
 
+#if P_SDL
+    PSDLDisplayThread *sdlThread;
+    PMutex           sdlThreadLock;
+#endif
+    int channelsOpenLimit;
+
+    BOOL behind_masq;
+    PIPSocket::Address *masqAddressPtr;
+
   friend class MyH323Connection;
 };
 
@@ -495,7 +588,14 @@ class MyH323Connection : public H323Connection
     PCLASSINFO(MyH323Connection, H323Connection);
 
   public:
-    MyH323Connection(MyH323EndPoint &, unsigned, BOOL, BOOL, WORD, int);
+    MyH323Connection(
+      MyH323EndPoint & ep,
+      unsigned callRef,
+      unsigned options,
+      unsigned minJitter,
+      unsigned maxJitter,
+      int verbose
+    );
 
     // overrides from H323Connection
     BOOL OnSendSignalSetup(H323SignalPDU & setupPDU);
@@ -512,6 +612,7 @@ class MyH323Connection : public H323Connection
     int channelsOpen;
 };
 
+//////////////////////////////////////////////
 
 class UserInterfaceThread : public PThread
 {
@@ -525,6 +626,19 @@ class UserInterfaceThread : public PThread
     MyH323EndPoint & endpoint;
 };
 
+//////////////////////////////////////////////
+
+class TestUserInterfaceThread : public PThread
+{
+    PCLASSINFO(TestUserInterfaceThread, PThread);
+  public:
+    TestUserInterfaceThread(MyH323EndPoint & end)
+      : PThread(1000, NoAutoDeleteThread), endpoint(end) { Resume(); }
+    void Main()
+      { endpoint.TestHandleUserInterface(); }
+  protected:
+    MyH323EndPoint & endpoint;
+};
 
 #endif  // _OhPhone_MAIN_H
 
